package com.docus.server.collect.dept.http;

import com.docus.core.util.json.JSON;
import com.docus.infrastructure.web.api.CommonResult;
import com.docus.log.annotation.TrackGroup;
import com.docus.log.context.TrackHelper;
import com.docus.server.collect.IConverter;
import com.docus.server.collect.IHttpResult;
import com.docus.server.enums.CollectTypeEnum;
import com.docus.server.collect.web.service.CollectService;
import com.docus.server.sys.common.pojo.dto.DeptDTO;
import com.docus.server.collect.web.process.VisitorProcessor;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;


/**
 * @author wen yongbin
 * @date 2023年2月25日21:56:33
 * @see CollectTypeEnum 枚举
 * @see IConverter 通用转化器
 * @see IHttpResult 通用返回结果
 * @see VisitorProcessor 通用处理器
 */

@Api(value = "科室管理接口", tags = "科室管理接口")
@Slf4j
@RestController
@RequestMapping("/api/rest/dept")
public class DeptRestController {
    @Resource
    private CollectService collectService;

    /**
     * 被动http 科室信息接收，进行操作
     *
     * @param message 接收到的科室参数
     * @return 返回结果
     */
    @ApiOperation("新增/编辑科室(根据科室代码)")
    @PostMapping("/update")
    @TrackGroup(
        group = "HTTP_PASSIVE_DEPT", retryKey = "dept",
        beanNames = {"httpDeptConverter", "httpDeptResultImpl"},
        processor = VisitorProcessor.class)
    public CommonResult httpPassiveDept(@RequestBody String message) {
        log.info("新增/修改科室数据：{}", message);
        collectService.insertOrUpdateDept(JSON.fromJSON((String) TrackHelper.getValue("jsonStr"), DeptDTO.class));
        return null;
    }
}
