package com.docus.server.api.segmentation;

import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.dto.segmentation.ocrconfig.AddOcrConfigDTO;
import com.docus.server.dto.segmentation.ocrconfig.DeleteOcrConfigDTO;
import com.docus.server.dto.segmentation.ocrconfig.EditOcrConfigDTO;
import com.docus.server.vo.segmentation.ocrconfig.OcrConfigVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;


/**
 * 基础配置表 API
 *
 * @author AutoGenerator
 * @since 2023-08-25
 */
@FeignClient(value = "docus-segmentation", contextId = "docus-segmentation.OcrConfigApi")
@RequestMapping("/ocrConfig")
public interface OcrConfigApi {

    /**
     * 按主键查询
     *
     * @param id 主键id
     * @return 实体
     */
    @GetMapping("/find/{id}")
    OcrConfigVO findById(@PathVariable(value = "id") Long id);

    /**
     * 查询所有
     *
     * @return 实体
     */
    @GetMapping("/findAll")
    List<OcrConfigVO> findAll();

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @PostMapping("/search")
    PageResult<OcrConfigVO> search(@RequestBody SearchDTO searchDTO);

    /**
     * 新增
     *
     * @param addOcrConfigDTO 新增参数
     */
    @PostMapping("/add")
    void add(@RequestBody AddOcrConfigDTO addOcrConfigDTO);

    /**
     * 编辑
     *
     * @param editOcrConfigDTO 编辑参数
     */
    @PutMapping("/edit")
    void edit(@RequestBody EditOcrConfigDTO editOcrConfigDTO);

    /**
     * 批量删除
     *
     * @param deleteOcrConfigDTO 删除参数
     * @return 成功或失败
     */
    @DeleteMapping("/delete")
    int delete(@RequestBody DeleteOcrConfigDTO deleteOcrConfigDTO);
}
