package com.docus.server.api.segmentation;

import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.dto.segmentation.ocrrule.AddOcrRuleDTO;
import com.docus.server.dto.segmentation.ocrrule.DeleteOcrRuleDTO;
import com.docus.server.dto.segmentation.ocrrule.EditOcrRuleDTO;
import com.docus.server.vo.segmentation.ocrrule.OcrRuleVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;


/**
 * 匹配策略表 API
 *
 * @author AutoGenerator
 * @since 2023-08-25
 */
@FeignClient(value = "docus-segmentation", contextId = "docus-segmentation.OcrRuleApi")
@RequestMapping("/ocrRule")
public interface OcrRuleApi {

    /**
     * 按主键查询
     *
     * @param id 主键id
     * @return 实体
     */
    @GetMapping("/find/{id}")
    OcrRuleVO findById(@PathVariable(value = "id") Long id);

    /**
     * 查询所有
     *
     * @return 实体
     */
    @GetMapping("/findAll")
    List<OcrRuleVO> findAll();

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @PostMapping("/search")
    PageResult<OcrRuleVO> search(@RequestBody SearchDTO searchDTO);

    /**
     * 新增
     *
     * @param addOcrRuleDTO 新增参数
     */
    @PostMapping("/add")
    void add(@RequestBody AddOcrRuleDTO addOcrRuleDTO);

    /**
     * 编辑
     *
     * @param editOcrRuleDTO 编辑参数
     */
    @PutMapping("/edit")
    void edit(@RequestBody EditOcrRuleDTO editOcrRuleDTO);

    /**
     * 批量删除
     *
     * @param deleteOcrRuleDTO 删除参数
     * @return 成功或失败
     */
    @DeleteMapping("/delete")
    int delete(@RequestBody DeleteOcrRuleDTO deleteOcrRuleDTO);
}
