package com.docus.server.api.segmentation;

import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.dto.segmentation.ocrsprule.AddOcrSpRuleDTO;
import com.docus.server.dto.segmentation.ocrsprule.DeleteOcrSpRuleDTO;
import com.docus.server.dto.segmentation.ocrsprule.EditOcrSpRuleDTO;
import com.docus.server.vo.segmentation.ocrsprule.OcrSpRuleVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.List;


/**
 * 特殊策略表 API
 *
 * @author AutoGenerator
 * @since 2023-08-21
 */
@FeignClient(value = "docus-segmentation", contextId = "docus-segmentation.OcrSpRuleApi")
@RequestMapping("/ocrSpRule")
public interface OcrSpRuleApi {

    /**
     * 按主键查询
     *
     * @param id 主键id
     * @return 实体
     */
    @GetMapping("/find/{id}")
    OcrSpRuleVO findById(@PathVariable(value = "id") Long id);

    /**
     * 查询所有
     *
     * @return 实体
     */
    @GetMapping("/findAll")
    List<OcrSpRuleVO> findAll();

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @PostMapping("/search")
    PageResult<OcrSpRuleVO> search(@RequestBody SearchDTO searchDTO);

    /**
     * 新增
     *
     * @param addOcrSpRuleDTO 新增参数
     */
    @PostMapping("/add")
    boolean add(@RequestBody AddOcrSpRuleDTO addOcrSpRuleDTO);

    /**
     * 编辑
     *
     * @param editOcrSpRuleDTO 编辑参数
     */
    @PutMapping("/edit")
    boolean edit(@RequestBody EditOcrSpRuleDTO editOcrSpRuleDTO);

    /**
     * 批量删除
     *
     * @param deleteOcrSpRuleDTO 删除参数
     * @return 成功或失败
     */
    @DeleteMapping("/delete")
    int delete(@RequestBody DeleteOcrSpRuleDTO deleteOcrSpRuleDTO);
}
