package com.docus.server.infrastructure.dao.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.docus.infrastructure.core.db.dao.impl.BaseDaoImpl;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.entity.segmentation.OcrVersion;
import com.docus.server.infrastructure.dao.IOcrVersionDao;
import com.docus.server.infrastructure.mapper.OcrVersionMapper;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;

/**
 * 正式数据版本号管理 数据访问实现类
 *
 * @author AutoGenerator
 * @since 2023-08-25
 */
@Repository
public class OcrVersionDaoImpl extends BaseDaoImpl<OcrVersionMapper, OcrVersion> implements IOcrVersionDao {

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public OcrVersion findById(Long id) {
        return super.findById(id);
    }

    /**
     * 新增
     *
     * @param ocrVersion 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(OcrVersion ocrVersion) {
        return super.save(ocrVersion);
    }

    /**
     * 编辑
     *
     * @param ocrVersion 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(OcrVersion ocrVersion) {
        return super.updateById(ocrVersion);
    }

    /**
     * 批量删除
     *
     * @param ids 主键ids
     * @return 成功或失败
     */
    @Override
    public int delete(List<Long> ids) {
        return super.deleteByIds(ids);
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<OcrVersion> search(SearchDTO searchDTO) {
        //分页
        PageHelper.startPage(searchDTO.getPageNum(), searchDTO.getPageSize());
        LambdaQueryWrapper<OcrVersion> query = Wrappers.lambdaQuery();
        //时间范围，默认按createTime
        if (searchDTO.getBeginTime() != null) {
            query.ge(OcrVersion::getCreateTime, searchDTO.getBeginTime());
        }
        if (searchDTO.getEndTime() != null) {
            Date endTime = searchDTO.getEndTime();
            query.le(OcrVersion::getCreateTime, endTime);
        }
        //关键字比较，多个列or
        //if (!StringUtils.isEmpty(searchDTO.getKeyword())) {
        //单个列用like
        //query. like(OcrVersion::getxxx, searchDTO.getKeyword());
        //多个列用like
        //query. and(sub -> sub.like(OcrVersion::getxx1, searchDTO.getKeyword())
        // оr(). like(OcrVersion::getXX2, searchDTO.getKeyword()))
        //);
        //}
        //默认createTime倒序排序
        query.orderByDesc(OcrVersion::getCreateTime);
        List<OcrVersion> list = super.find(query);
        return new PageResult<>(list);
    }

    /**
     * 名称不重复
     *
     * @param id   主键
     * @param name 名称
     * @return 名称重复数量
     */
    @Override
    public int findByIdAndName(Long id, String name) {
        LambdaQueryWrapper<OcrVersion> query = Wrappers.lambdaQuery();
        if (id != null) {
            query.ne(OcrVersion::getId, id);
        }
        //if (Func.isNotBlank(name)) {
        //   query.eq(OcrVersion::getName, name);
        //}

        return baseMapper.selectCount(query);
    }
}
