package com.docus.server.service.impl;

import com.docus.infrastructure.redis.service.IdService;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.OcrConfigConvert;
import com.docus.server.dto.segmentation.ocrconfig.AddOcrConfigDTO;
import com.docus.server.dto.segmentation.ocrconfig.DeleteOcrConfigDTO;
import com.docus.server.dto.segmentation.ocrconfig.EditOcrConfigDTO;
import com.docus.server.entity.segmentation.OcrConfig;
import com.docus.server.infrastructure.dao.IOcrConfigDao;
import com.docus.server.service.IOcrConfigService;
import com.docus.server.vo.segmentation.ocrconfig.OcrConfigVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 基础配置表 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-08-25
 */
@Service
public class OcrConfigServiceImpl implements IOcrConfigService {
    @Resource
    private IOcrConfigDao iOcrConfigDao;
    @Resource
    private IdService idService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public OcrConfigVO findById(Long id) {
        return OcrConfigConvert.INSTANCE.convertVO(iOcrConfigDao.findById(id));
    }

    /**
     * 查询所有
     *
     * @return 实体
     */
    @Override
    public List<OcrConfigVO> findAll() {
        return OcrConfigConvert.INSTANCE.convertVO(iOcrConfigDao.findAll());
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<OcrConfigVO> search(SearchDTO searchDTO) {
        return OcrConfigConvert.INSTANCE.convertVO(iOcrConfigDao.search(searchDTO));

        //PageResult<OcrConfigVO> result = OcrConfigConvert.INSTANCE.convertVO(iOcrConfigDao.search(searchDTO));

        //if (CollectionUtils.isEmpty(result.getList())) {
        //return new PageResult<>();
        //}

        //Map<String, SchSystemParams> map = iSchSystemParamsService.find(ListUtils.distinctSelect(result.getList(), SchCollectorVO::getCollectorId));

        //result.getList().forEach(p -> {
        // String collectorId = String.valueOf(p.getCollectorId());
        // if (map.containsKey(collectorId)) {
        //     p.setCollectorName(map.get(collectorId).getParamName());
        // }
        //});

        //return result;
    }

    /**
     * 新增
     *
     * @param addOcrConfigDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean add(AddOcrConfigDTO addOcrConfigDTO) {
        OcrConfig ocrConfig = OcrConfigConvert.INSTANCE.convertDO(addOcrConfigDTO);
        ocrConfig.setId(idService.getDateSeq());
        return iOcrConfigDao.add(ocrConfig);

        // String name = addOcrConfigDTO.getName();

        //OcrConfig ocrConfig = iOcrConfigDao.findOneBy("name", name);

        //if (Func.notNull(ocrConfig)) {
        // throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "基础配置表已经存在");
        // }

        // OcrConfig ocrConfig =OcrConfigConvert.INSTANCE.convertDO(addOcrConfigDTO);
        // ocrConfig.setId(idService.getDateSeq());

        // return iOcrConfigDao.add(schCollector);

    }

    /**
     * 编辑
     *
     * @param editOcrConfigDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean edit(EditOcrConfigDTO editOcrConfigDTO) {
        OcrConfig ocrConfig = OcrConfigConvert.INSTANCE.convertDO(editOcrConfigDTO);
        return iOcrConfigDao.edit(ocrConfig);

        // Long id = editOcrConfigDTO.getId();

        //  OcrConfig ocrConfig = iOcrConfigDao.findById(id);

        // if (Func.isNull(ocrConfig)) {
        //     throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "基础配置表，无法更新！");
        // }

        // ocrConfig.setUpdateTime(LocalDateTime.now());
        // return iOcrConfigDao.edit(ocrConfig);
    }

    /**
     * 批量删除
     *
     * @param deleteOcrConfigDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteOcrConfigDTO deleteOcrConfigDTO) {
        return iOcrConfigDao.delete(deleteOcrConfigDTO.getIds());
    }
}

