package com.docus.server.service.impl;

import com.docus.infrastructure.redis.service.IdService;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.OcrFileInfoConvert;
import com.docus.server.dto.segmentation.ocrfileinfo.AddOcrFileInfoDTO;
import com.docus.server.dto.segmentation.ocrfileinfo.DeleteOcrFileInfoDTO;
import com.docus.server.dto.segmentation.ocrfileinfo.EditOcrFileInfoDTO;
import com.docus.server.entity.segmentation.OcrFileInfo;
import com.docus.server.infrastructure.dao.IOcrFileInfoDao;
import com.docus.server.service.IOcrFileInfoService;
import com.docus.server.vo.segmentation.ocrfileinfo.OcrFileInfoVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * ocr文件信息 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-08-25
 */
@Service
public class OcrFileInfoServiceImpl implements IOcrFileInfoService {
    @Resource
    private IOcrFileInfoDao iOcrFileInfoDao;
    @Resource
    private IdService idService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public OcrFileInfoVO findById(Long id) {
        return OcrFileInfoConvert.INSTANCE.convertVO(iOcrFileInfoDao.findById(id));
    }

    /**
     * 查询所有
     *
     * @return 实体
     */
    @Override
    public List<OcrFileInfoVO> findAll() {
        return OcrFileInfoConvert.INSTANCE.convertVO(iOcrFileInfoDao.findAll());
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<OcrFileInfoVO> search(SearchDTO searchDTO) {
        return OcrFileInfoConvert.INSTANCE.convertVO(iOcrFileInfoDao.search(searchDTO));

        //PageResult<OcrFileInfoVO> result = OcrFileInfoConvert.INSTANCE.convertVO(iOcrFileInfoDao.search(searchDTO));

        //if (CollectionUtils.isEmpty(result.getList())) {
        //return new PageResult<>();
        //}

        //Map<String, SchSystemParams> map = iSchSystemParamsService.find(ListUtils.distinctSelect(result.getList(), SchCollectorVO::getCollectorId));

        //result.getList().forEach(p -> {
        // String collectorId = String.valueOf(p.getCollectorId());
        // if (map.containsKey(collectorId)) {
        //     p.setCollectorName(map.get(collectorId).getParamName());
        // }
        //});

        //return result;
    }

    /**
     * 新增
     *
     * @param addOcrFileInfoDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean add(AddOcrFileInfoDTO addOcrFileInfoDTO) {
        OcrFileInfo ocrFileInfo = OcrFileInfoConvert.INSTANCE.convertDO(addOcrFileInfoDTO);
        ocrFileInfo.setId(idService.getDateSeq());
        return iOcrFileInfoDao.add(ocrFileInfo);

        // String name = addOcrFileInfoDTO.getName();

        //OcrFileInfo ocrFileInfo = iOcrFileInfoDao.findOneBy("name", name);

        //if (Func.notNull(ocrFileInfo)) {
        // throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "ocr文件信息已经存在");
        // }

        // OcrFileInfo ocrFileInfo =OcrFileInfoConvert.INSTANCE.convertDO(addOcrFileInfoDTO);
        // ocrFileInfo.setId(idService.getDateSeq());

        // return iOcrFileInfoDao.add(schCollector);

    }

    /**
     * 编辑
     *
     * @param editOcrFileInfoDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean edit(EditOcrFileInfoDTO editOcrFileInfoDTO) {
        OcrFileInfo ocrFileInfo = OcrFileInfoConvert.INSTANCE.convertDO(editOcrFileInfoDTO);
        return iOcrFileInfoDao.edit(ocrFileInfo);

        // Long id = editOcrFileInfoDTO.getId();

        //  OcrFileInfo ocrFileInfo = iOcrFileInfoDao.findById(id);

        // if (Func.isNull(ocrFileInfo)) {
        //     throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "ocr文件信息，无法更新！");
        // }

        // ocrFileInfo.setUpdateTime(LocalDateTime.now());
        // return iOcrFileInfoDao.edit(ocrFileInfo);
    }

    /**
     * 批量删除
     *
     * @param deleteOcrFileInfoDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteOcrFileInfoDTO deleteOcrFileInfoDTO) {
        return iOcrFileInfoDao.delete(deleteOcrFileInfoDTO.getIds());
    }
}

