package com.docus.server.service.impl;

import com.docus.infrastructure.redis.service.IdService;
import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.server.convert.OcrSpRuleConvert;
import com.docus.server.dto.segmentation.ocrsprule.AddOcrSpRuleDTO;
import com.docus.server.dto.segmentation.ocrsprule.DeleteOcrSpRuleDTO;
import com.docus.server.dto.segmentation.ocrsprule.EditOcrSpRuleDTO;
import com.docus.server.entity.segmentation.OcrSpRule;
import com.docus.server.infrastructure.dao.IOcrSpRuleDao;
import com.docus.server.service.IOcrSpRuleService;
import com.docus.server.vo.segmentation.ocrsprule.OcrSpRuleVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 特殊策略表 服务实现类
 *
 * @author AutoGenerator
 * @since 2023-08-21
 */
@Service
public class OcrSpRuleServiceImpl implements IOcrSpRuleService {
    @Resource
    private IOcrSpRuleDao iOcrSpRuleDao;
    @Resource
    private IdService idService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public OcrSpRuleVO findById(Long id) {
        return OcrSpRuleConvert.INSTANCE.convertVO(iOcrSpRuleDao.findById(id));
    }

    /**
     * 查询所有
     *
     * @return 实体
     */
    @Override
    public List<OcrSpRuleVO> findAll() {
        return OcrSpRuleConvert.INSTANCE.convertVO(iOcrSpRuleDao.findAll());
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<OcrSpRuleVO> search(SearchDTO searchDTO) {
        return OcrSpRuleConvert.INSTANCE.convertVO(iOcrSpRuleDao.search(searchDTO));

        //PageResult<OcrSpRuleVO> result = OcrSpRuleConvert.INSTANCE.convertVO(iOcrSpRuleDao.search(searchDTO));

        //if (CollectionUtils.isEmpty(result.getList())) {
        //return new PageResult<>();
        //}

        //Map<String, SchSystemParams> map = iSchSystemParamsService.find(ListUtils.distinctSelect(result.getList(), SchCollectorVO::getCollectorId));

        //result.getList().forEach(p -> {
        // String collectorId = String.valueOf(p.getCollectorId());
        // if (map.containsKey(collectorId)) {
        //     p.setCollectorName(map.get(collectorId).getParamName());
        // }
        //});

        //return result;
    }

    /**
     * 新增
     *
     * @param addOcrSpRuleDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean add(AddOcrSpRuleDTO addOcrSpRuleDTO) {
        OcrSpRule ocrSpRule = OcrSpRuleConvert.INSTANCE.convertDO(addOcrSpRuleDTO);
        ocrSpRule.setId(idService.getDateSeq());
        return iOcrSpRuleDao.add(ocrSpRule);

        // String name = addOcrSpRuleDTO.getName();

        //OcrSpRule ocrSpRule = iOcrSpRuleDao.findOneBy("name", name);

        //if (Func.notNull(ocrSpRule)) {
        // throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "特殊策略表已经存在");
        // }

        // OcrSpRule ocrSpRule =OcrSpRuleConvert.INSTANCE.convertDO(addOcrSpRuleDTO);
        // ocrSpRule.setId(idService.getDateSeq());

        // return iOcrSpRuleDao.add(schCollector);

    }

    /**
     * 编辑
     *
     * @param editOcrSpRuleDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean edit(EditOcrSpRuleDTO editOcrSpRuleDTO) {
        OcrSpRule ocrSpRule = OcrSpRuleConvert.INSTANCE.convertDO(editOcrSpRuleDTO);
        return iOcrSpRuleDao.edit(ocrSpRule);

        // Long id = editOcrSpRuleDTO.getId();

        //  OcrSpRule ocrSpRule = iOcrSpRuleDao.findById(id);

        // if (Func.isNull(ocrSpRule)) {
        //     throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "特殊策略表，无法更新！");
        // }

        // ocrSpRule.setUpdateTime(LocalDateTime.now());
        // return iOcrSpRuleDao.edit(ocrSpRule);
    }

    /**
     * 批量删除
     *
     * @param deleteOcrSpRuleDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteOcrSpRuleDTO deleteOcrSpRuleDTO) {
        return iOcrSpRuleDao.delete(deleteOcrSpRuleDTO.getIds());
    }
}

